@@ -1,7 +1,8 @@
 package SVN::Look;
 
-use warnings;
 use strict;
+use warnings;
+use Carp;
 use File::Spec::Functions qw/catfile path rootdir/;
 
 =head1 NAME
@@ -10,11 +11,11 @@ SVN::Look - A caching wrapper aroung the svnlook command.
 
 =head1 VERSION
 
-Version 0.13
+Version 0.18
 
 =cut
 
-our $VERSION = '0.14.' . substr(q$Revision: 10 $, 10); # bump from 5
+our $VERSION = '0.18';
 
 =head1 SYNOPSIS
 
@@ -52,8 +53,8 @@ for my $d (
 ) {
     my $f = catfile($d, 'svnlook');
     if (-x $f) {
-	$SVNLOOK = $f;
-	last;
+        $SVNLOOK = $f;
+        last;
     }
 }
 die "Aborting because I couldn't find the svnlook executable.\n"
@@ -84,23 +85,23 @@ by WHAT.
 sub new {
     my ($class, $repo, $what, $txn_or_rev) = @_;
     my $self = {
-	repo     => $repo,
-	what     => [$what, $txn_or_rev],
-	txn      => undef,
-	rev      => undef,
-	author   => undef,
-	log      => undef,
-	changed  => undef,
-	proplist => undef,
+        repo     => $repo,
+        what     => [$what, $txn_or_rev],
+        txn      => undef,
+        rev      => undef,
+        author   => undef,
+        log      => undef,
+        changed  => undef,
+        proplist => undef,
     };
     if ($what eq '-t') {
-	$self->{txn} = $txn_or_rev;
+        $self->{txn} = $txn_or_rev;
     }
     elsif ($what eq '-r') {
-	$self->{rev} = $txn_or_rev;
+        $self->{rev} = $txn_or_rev;
     }
     else {
-	die "Look::new: third argument must be -t or -r, not ($what)";
+        croak "Look::new: third argument must be -t or -r, not ($what)";
     }
     bless $self, $class;
     return $self;
@@ -108,20 +109,22 @@ sub new {
 
 sub _svnlook {
     my ($self, $cmd, @args) = @_;
-    open my $fd, '-|', $SVNLOOK, $cmd, $self->{repo}, @{$self->{what}}, @args
-	or die "Can't exec svnlook $cmd: $!\n";
+    my @cmd = ($SVNLOOK, $cmd, $self->{repo});
+    push @cmd, @{$self->{what}} unless $cmd =~ /^(?:youngest|uuid|lock)$/;
+    open my $fd, '-|', @cmd, @args
+        or die "Can't exec svnlook $cmd: $!\n";
     if (wantarray) {
-	my @lines = <$fd>;
-	close $fd or die "Failed closing svnlook $cmd: $!\n";
-	chomp foreach @lines;
-	return @lines;
+        my @lines = <$fd>;
+        close $fd or die "Failed closing svnlook $cmd: $!\n";
+        chomp foreach @lines;
+        return @lines;
     }
     else {
-	local $/ = undef;
-	my $line = <$fd>;
-	close $fd or die "Failed closing svnlook $cmd: $!\n";
-	chomp $line;
-	return $line;
+        local $/ = undef;
+        my $line = <$fd>;
+        close $fd or die "Failed closing svnlook $cmd: $!\n";
+        chomp $line;
+        return $line;
     }
 }
 
@@ -169,7 +172,7 @@ Returns the author of the revision/transaction.
 sub author {
     my $self = shift;
     unless ($self->{author}) {
-	chomp($self->{author} = $self->_svnlook('author'));
+        chomp($self->{author} = $self->_svnlook('author'));
     }
     return $self->{author};
 }
@@ -183,7 +186,7 @@ Returns the log message of the revision/transaction.
 sub log_msg {
     my $self = shift;
     unless ($self->{log}) {
-	$self->{log} = $self->_svnlook('log');
+        $self->{log} = $self->_svnlook('log');
     }
     return $self->{log};
 }
@@ -197,7 +200,7 @@ Returns the date of the revision/transaction.
 sub date {
     my $self = shift;
     unless ($self->{date}) {
-	$self->{date} = ($self->_svnlook('info'))[1];
+        $self->{date} = ($self->_svnlook('info'))[1];
     }
     return $self->{date};
 }
@@ -211,11 +214,11 @@ Returns a reference to a hash containing the properties associated with PATH.
 sub proplist {
     my ($self, $path) = @_;
     unless ($self->{proplist}{$path}) {
-	my $text = $self->_svnlook('proplist', '--verbose', $path);
-	my @list = split /^\s\s(\S+)\s:\s/m, $text;
-	shift @list;		# skip the leading empty field
-	chomp(my %hash = @list);
-	$self->{proplist}{$path} = \%hash;
+        my $text = $self->_svnlook('proplist', '--verbose', $path);
+        my @list = split /^\s\s(\S+)\s:\s/m, $text;
+        shift @list;            # skip the leading empty field
+        chomp(my %hash = @list);
+        $self->{proplist}{$path} = \%hash;
     }
     return $self->{proplist}{$path};
 }
@@ -256,36 +259,36 @@ revision.
 sub changed_hash {
     my $self = shift;
     unless ($self->{changed_hash}) {
-	my (@added, @deleted, @updated, @prop_modified, %copied);
-	foreach ($self->_svnlook('changed', '--copy-info')) {
-	    next if length($_) <= 4;
-	    chomp;
-	    my ($action, $prop, undef, undef, $changed) = unpack 'AAAA A*', $_;
-	    if    ($action eq 'A') {
-		push @added,   $changed;
-	    }
-	    elsif ($action eq 'D') {
-		push @deleted, $changed;
-	    }
-	    elsif ($action eq 'U') {
-		push @updated, $changed;
-	    }
-	    else {
-		if ($changed =~ /^\(from (.*?):r(\d+)\)$/) {
-		    $copied{$added[-1]} = [$1 => $2];
-		}
-	    }
-	    if ($prop eq 'U') {
-		push @prop_modified, $changed;
-	    }
-	}
-	$self->{changed_hash} = {
-	    added         => \@added,
-	    deleted       => \@deleted,
-	    updated       => \@updated,
-	    prop_modified => \@prop_modified,
-	    copied        => \%copied,
-	};
+        my (@added, @deleted, @updated, @prop_modified, %copied);
+        foreach ($self->_svnlook('changed', '--copy-info')) {
+            next if length($_) <= 4;
+            chomp;
+            my ($action, $prop, undef, undef, $changed) = unpack 'AAAA A*', $_;
+            if    ($action eq 'A') {
+                push @added,   $changed;
+            }
+            elsif ($action eq 'D') {
+                push @deleted, $changed;
+            }
+            elsif ($action eq 'U') {
+                push @updated, $changed;
+            }
+            else {
+                if ($changed =~ /^\(from (.*?):r(\d+)\)$/) {
+                    $copied{$added[-1]} = [$1 => $2];
+                }
+            }
+            if ($prop eq 'U') {
+                push @prop_modified, $changed;
+            }
+        }
+        $self->{changed_hash} = {
+            added         => \@added,
+            deleted       => \@deleted,
+            updated       => \@updated,
+            prop_modified => \@prop_modified,
+            copied        => \%copied,
+        };
     }
     return $self->{changed_hash};
 }
@@ -346,7 +349,7 @@ sub changed {
     my $self = shift;
     my $hash = $self->changed_hash();
     unless (exists $hash->{changed}) {
-	$hash->{changed} = [@{$hash->{added}}, @{$hash->{updated}}, @{$hash->{deleted}}, @{$hash->{prop_modified}}];
+        $hash->{changed} = [@{$hash->{added}}, @{$hash->{updated}}, @{$hash->{deleted}}, @{$hash->{prop_modified}}];
     }
     return @{$hash->{changed}};
 }
@@ -360,8 +363,8 @@ Returns the list of directories changed in the revision/transaction.
 sub dirs_changed {
     my $self = shift;
     unless (exists $self->{dirs_changed}) {
-	my @dirs = $self->_svnlook('dirs-changed');
-	$self->{dirs_changed} = \@dirs;
+        my @dirs = $self->_svnlook('dirs-changed');
+        $self->{dirs_changed} = \@dirs;
     }
     return @{$self->{dirs_changed}};
 }
@@ -435,6 +438,76 @@ sub diff {
     return $self->_svnlook('diff', @opts);
 }
 
+=item B<youngest>
+
+Returns the repository's youngest revision number.
+
+=cut
+
+sub youngest {
+    my ($self) = @_;
+    return $self->_svnlook('youngest');
+}
+
+=item B<uuid>
+
+Returns the repository's UUID.
+
+=cut
+
+sub uuid {
+    my ($self) = @_;
+    return $self->_svnlook('uuid');
+}
+
+=item B<lock> PATH
+
+If PATH has a lock, returns a hash containing information about the lock, with the following keys:
+
+=over
+
+=item UUID Token
+
+A string with the opaque lock token.
+
+=item Owner
+
+The name of the user that has the lock.
+
+=item Created
+
+The time at which the lock was created, in a format like this: '2010-02-16 17:23:08 -0200 (Tue, 16 Feb 2010)'.
+
+=item Comment
+
+The lock comment.
+
+=back
+
+If PATH has no lock, returns undef.
+
+=cut
+
+sub lock {
+    my ($self, $path) = @_;
+    my %lock = ();
+    my @lock = $self->_svnlook('lock', $path);
+
+    while (my $line = shift @lock) {
+	chomp $line;
+	my ($key, $value) = split /:\s*/, $line, 2;
+	if ($key =~ /^Comment/) {
+	    $lock{Comment} = join('', @lock);
+	    last;
+	}
+	else {
+	    $lock{$key} = $value;
+	}
+    }
+
+    return %lock ? \%lock : undef;
+}
+
 =back
 
 =head1 AUTHOR
@@ -477,7 +550,7 @@ L<http://search.cpan.org/dist/SVN-Hooks>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2008 CPqD, all rights reserved.
+Copyright 2008-2010 CPqD, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
